'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _sequelize = require('sequelize');

var _sequelize2 = _interopRequireDefault(_sequelize);

var _graphql = require('graphql');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

module.exports = function (target, options) {
  var resolver, targetAttributes, argsToFindOptions;

  targetAttributes = target instanceof _sequelize2['default'].Model ? _Object$keys(target.rawAttributes) : _Object$keys(target.target.rawAttributes);

  argsToFindOptions = function (args) {
    var result = {};

    if (args) {
      _Object$keys(args).forEach(function (key) {
        if (~targetAttributes.indexOf(key)) {
          result.where = result.where || {};
          result.where[key] = args[key];
        }

        if (key === 'limit' && args[key]) {
          result.limit = args[key];
        }

        if (key === 'order' && args[key]) {
          result.order = [[args[key]]];
        }
      });
    }

    result.logging = options.logging;
    return result;
  };

  options = options || {};
  if (options.include === undefined) options.include = true;
  if (options.before === undefined) options.before = function (options) {
    return options;
  };

  if (target instanceof _sequelize2['default'].Model) {
    resolver = function (source, args, root, ast, type) {
      var selections,
          attributes,
          include = [],
          list = type instanceof _graphql.GraphQLList,
          findOptions = argsToFindOptions(args);

      type = type.ofType || type;

      selections = ast.selectionSet.selections.reduce(function (memo, selection) {
        memo[selection.name.value] = selection;
        return memo;
      }, {});

      attributes = _Object$keys(selections).filter(function (attribute) {
        return ~targetAttributes.indexOf(attribute);
      });

      if (! ~attributes.indexOf(target.primaryKeyAttribute)) {
        attributes.push(target.primaryKeyAttribute);
      }

      _Object$keys(selections).forEach(function (key) {
        var association, includeOptions, args;

        association = type._fields[key].resolve && type._fields[key].resolve.$association;

        if (association) {
          args = selections[key].arguments.reduce(function (memo, arg) {
            memo[arg.name.value] = arg.value.value;
            return memo;
          }, {});

          includeOptions = argsToFindOptions(args);

          if (options.include && !includeOptions.limit) {
            if (includeOptions.order) {
              includeOptions.order.map(function (order) {
                order.unshift({
                  model: association.target,
                  as: association.options.as
                });

                return order;
              });

              findOptions.order = (findOptions.order || []).concat(includeOptions.order);

              delete includeOptions.order;
            }

            include.push(_lodash2['default'].assign({ association: association }, includeOptions));
          } else if (association.associationType === 'BelongsTo') {
            if (! ~attributes.indexOf(association.foreignKey)) {
              attributes.push(association.foreignKey);
            }
          }
        }
      });

      findOptions.include = include;
      findOptions.attributes = attributes;
      findOptions.root = root;

      return target[list ? 'findAll' : 'findOne'](options.before(findOptions, root));
    };
  }

  if (target instanceof require('sequelize/lib/associations/base')) {
    resolver = function (source, args, root) {
      if (source.get(target.as)) {
        return source.get(target.as);
      }

      var findOptions = argsToFindOptions(args);
      findOptions.root = root;

      return source[target.accessors.get](options.before(findOptions, root));
    };

    resolver.$association = target;
  }

  return resolver;
};